/**
 * @file
 * VuoList_VuoHorizontalReflection C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoHorizontalReflection_H
#define VuoList_VuoHorizontalReflection_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoHorizontalReflection VuoList_VuoHorizontalReflection
 * A list of @ref VuoHorizontalReflection elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoHorizontalReflection_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoHorizontalReflection elements.
 */
typedef const struct VuoList_VuoHorizontalReflection_struct { void *l; } * VuoList_VuoHorizontalReflection;
/// @}
#endif

/**
 * Creates a new list of @ref VuoHorizontalReflection elements.
 */
VuoList_VuoHorizontalReflection VuoListCreate_VuoHorizontalReflection(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoHorizontalReflection VuoListCopy_VuoHorizontalReflection(const VuoList_VuoHorizontalReflection list);

/**
 * Returns the @ref VuoHorizontalReflection at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoHorizontalReflection VuoListGetValue_VuoHorizontalReflection(const VuoList_VuoHorizontalReflection list, const unsigned long index);

/**
 * Changes the @ref VuoHorizontalReflection at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoHorizontalReflection(const VuoList_VuoHorizontalReflection list, const VuoHorizontalReflection value, const unsigned long index);

/**
 * Inserts the `VuoHorizontalReflection` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoHorizontalReflection(const VuoList_VuoHorizontalReflection list, const VuoHorizontalReflection value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const VuoHorizontalReflection value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const VuoHorizontalReflection value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list);

/**
 * Removes the `VuoHorizontalReflection` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoHorizontalReflection(VuoList_VuoHorizontalReflection list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoHorizontalReflection(const VuoList_VuoHorizontalReflection list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoHorizontalReflection VuoList_VuoHorizontalReflection_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoHorizontalReflection_getJson(const VuoList_VuoHorizontalReflection value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoHorizontalReflection_getSummary(const VuoList_VuoHorizontalReflection value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoHorizontalReflection VuoList_VuoHorizontalReflection_makeFromString(const char *str);
char * VuoList_VuoHorizontalReflection_getString(const VuoList_VuoHorizontalReflection value);
/// @}

/**
 * @}
 */

#endif
