/**
 * @file
 * VuoList_VuoLeapHand C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoLeapHand_H
#define VuoList_VuoLeapHand_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoLeapHand VuoList_VuoLeapHand
 * A list of @ref VuoLeapHand elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoLeapHand_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoLeapHand elements.
 */
typedef const struct VuoList_VuoLeapHand_struct { void *l; } * VuoList_VuoLeapHand;
/// @}
#endif

/**
 * Creates a new list of @ref VuoLeapHand elements.
 */
VuoList_VuoLeapHand VuoListCreate_VuoLeapHand(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoLeapHand VuoListCopy_VuoLeapHand(const VuoList_VuoLeapHand list);

/**
 * Returns the @ref VuoLeapHand at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoLeapHand VuoListGetValue_VuoLeapHand(const VuoList_VuoLeapHand list, const unsigned long index);

/**
 * Changes the @ref VuoLeapHand at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoLeapHand(const VuoList_VuoLeapHand list, const VuoLeapHand value, const unsigned long index);

/**
 * Inserts the `VuoLeapHand` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoLeapHand(const VuoList_VuoLeapHand list, const VuoLeapHand value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoLeapHand(VuoList_VuoLeapHand list, const VuoLeapHand value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoLeapHand(VuoList_VuoLeapHand list, const VuoLeapHand value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoLeapHand(VuoList_VuoLeapHand list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoLeapHand(VuoList_VuoLeapHand list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoLeapHand(VuoList_VuoLeapHand list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoLeapHand(VuoList_VuoLeapHand list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoLeapHand(VuoList_VuoLeapHand list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoLeapHand(VuoList_VuoLeapHand list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoLeapHand(VuoList_VuoLeapHand list);

/**
 * Removes the `VuoLeapHand` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoLeapHand(VuoList_VuoLeapHand list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoLeapHand(const VuoList_VuoLeapHand list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoLeapHand VuoList_VuoLeapHand_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoLeapHand_getJson(const VuoList_VuoLeapHand value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoLeapHand_getSummary(const VuoList_VuoLeapHand value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoLeapHand VuoList_VuoLeapHand_makeFromString(const char *str);
char * VuoList_VuoLeapHand_getString(const VuoList_VuoLeapHand value);
/// @}

/**
 * @}
 */

#endif
